@@ -220,7 +220,7 @@ def _cp_n_param(tensor_shape, rank, weights=False):
         return factors_params
 
 
-def _validate_cp_rank(tensor_shape, rank='same', rounding='round'):
+def validate_cp_rank(tensor_shape, rank='same', rounding='round'):
     """Returns the rank of a CP Decomposition
 
     Parameters
@@ -251,7 +251,7 @@ def _validate_cp_rank(tensor_shape, rank='same', rounding='round'):
     if rank == 'same':
         rank = float(1)
 
-    if isinstance(rank, float) and (0 < rank <= 1):
+    if isinstance(rank, float):
         rank = int(rounding_fun(np.prod(tensor_shape)*rank/np.sum(tensor_shape)))
     return rank
 
@@ -462,7 +462,11 @@ def cp_mode_dot(cp_tensor, matrix_or_vector, mode, keep_dim=False, copy=False):
         else:
              factors[mode] = T.dot(matrix_or_vector, factors[mode])
 
-        return CPTensor((weights, factors))
+        if copy:
+            return CPTensor((weights, factors))
+        else:
+            cp_tensor.shape = tuple(f.shape[0] for f in factors)
+            return cp_tensor
     
 
 def unfolding_dot_khatri_rao(tensor, cp_tensor, mode):
