@@ -7,7 +7,7 @@ from ..random import check_random_state, random_cp
 from ..base import unfold
 from ..cp_tensor import (cp_to_tensor, CPTensor,
                          unfolding_dot_khatri_rao, cp_norm,
-                         cp_normalize, _validate_cp_rank)
+                         cp_normalize, validate_cp_rank)
 
 # Authors: Jean Kossaifi <jean.kossaifi+tensors@gmail.com>
 #          Chris Swierczewski <csw@amazon.com>
@@ -46,7 +46,7 @@ def initialize_cp(tensor, rank, init='svd', svd='numpy_svd', random_state=None,
     if init == 'random':
         # factors = [tl.tensor(rng.random_sample((tensor.shape[i], rank)), **tl.context(tensor)) for i in range(tl.ndim(tensor))]
         # kt = CPTensor((None, factors))
-        return random_cp(tl.shape(tensor), rank, normalise_factors=False)
+        return random_cp(tl.shape(tensor), rank, normalise_factors=False, **tl.context(tensor))
 
     elif init == 'svd':
         try:
@@ -263,7 +263,7 @@ def parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_svd',\
     .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
             Applications", PhD., University of Amsterdam, 1998
     """
-    rank = _validate_cp_rank(tl.shape(tensor), rank=rank)
+    rank = validate_cp_rank(tl.shape(tensor), rank=rank)
     
     if orthogonalise and not isinstance(orthogonalise, int):
         orthogonalise = n_iter_max
@@ -465,7 +465,7 @@ def non_negative_parafac(tensor, rank, n_iter_max=100, init='svd', svd='numpy_sv
        pp 792-799, ICML, 2005
     """
     epsilon = 10e-12
-    rank = _validate_cp_rank(tl.shape(tensor), rank=rank)
+    rank = validate_cp_rank(tl.shape(tensor), rank=rank)
 
     if mask is not None and init == "svd":
         message = "Masking occurs after initialization. Therefore, random initialization is recommended."
@@ -671,7 +671,7 @@ def randomised_parafac(tensor, rank, n_samples, n_iter_max=100, init='random', s
     .. [3] Casey Battaglino, Grey Ballard and Tamara G. Kolda,
        "A Practical Randomized CP Tensor Decomposition",
     """
-    rank = _validate_cp_rank(tl.shape(tensor), rank=rank)
+    rank = validate_cp_rank(tl.shape(tensor), rank=rank)
 
     rng = check_random_state(random_state)
     weights, factors = initialize_cp(tensor, rank, init=init, svd=svd, random_state=random_state)
@@ -726,6 +726,78 @@ def randomised_parafac(tensor, rank, n_samples, n_iter_max=100, init='random', s
 
 
 class CP(DecompositionMixin):
+    """Candecomp-Parafac decomposition via Alternating-Least Square
+
+        Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
+
+            ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
+
+        Parameters
+        ----------
+        tensor : ndarray
+        rank  : int
+            Number of components.
+        n_iter_max : int
+            Maximum number of iteration
+        init : {'svd', 'random'}, optional
+            Type of factor matrix initialization. See `initialize_factors`.
+        svd : str, default is 'numpy_svd'
+            function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
+        normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
+            of shape (rank, ), which will contain the norms of the factors
+        tol : float, optional
+            (Default: 1e-6) Relative reconstruction error tolerance. The
+            algorithm is considered to have found the global minimum when the
+            reconstruction error is less than `tol`.
+        random_state : {None, int, np.random.RandomState}
+        verbose : int, optional
+            Level of verbosity
+        return_errors : bool, optional
+            Activate return of iteration errors
+        mask : ndarray
+            array of booleans with the same shape as ``tensor`` should be 0 where
+            the values are missing and 1 everywhere else. Note:  if tensor is
+            sparse, then mask should also be sparse with a fill value of 1 (or
+            True). Allows for missing values [2]_
+        cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
+            Stopping criterion for ALS, works if `tol` is not None. 
+            If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
+            If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
+        sparsity : float or int
+            If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
+        fixed_modes : list, default is []
+            A list of modes for which the initial value is not modified.
+            The last mode cannot be fixed due to error computation.
+        svd_mask_repeats: int
+            If using a tensor with masked values, this initializes using SVD multiple times to
+            remove the effect of these missing values on the initialization.
+        linesearch : bool, default is False
+            Whether to perform line search as proposed by Bro [3].
+
+        Returns
+        -------
+        CPTensor : (weight, factors)
+            * weights : 1D array of shape (rank, )
+                all ones if normalize_factors is False (default), 
+                weights of the (normalized) factors otherwise
+            * factors : List of factors of the CP decomposition element `i` is of shape
+                (tensor.shape[i], rank)
+            * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
+
+        errors : list
+            A list of reconstruction errors at each iteration of the algorithms.
+
+        References
+        ----------
+        .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
+        SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
+
+        .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
+                Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
+
+        .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
+                Applications", PhD., University of Amsterdam, 1998
+    """
     def __init__(self, rank, n_iter_max=100, tol=1e-08, 
                  init='svd', svd='numpy_svd',
                  l2_reg=0,
@@ -738,78 +810,6 @@ class CP(DecompositionMixin):
                  cvg_criterion='abs_rec_error',
                  random_state=None, 
                  verbose=0):
-        """Candecomp-Parafac decomposition via Alternating-Least Square
-
-            Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
-
-                ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
-
-            Parameters
-            ----------
-            tensor : ndarray
-            rank  : int
-                Number of components.
-            n_iter_max : int
-                Maximum number of iteration
-            init : {'svd', 'random'}, optional
-                Type of factor matrix initialization. See `initialize_factors`.
-            svd : str, default is 'numpy_svd'
-                function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-            normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
-                of shape (rank, ), which will contain the norms of the factors
-            tol : float, optional
-                (Default: 1e-6) Relative reconstruction error tolerance. The
-                algorithm is considered to have found the global minimum when the
-                reconstruction error is less than `tol`.
-            random_state : {None, int, np.random.RandomState}
-            verbose : int, optional
-                Level of verbosity
-            return_errors : bool, optional
-                Activate return of iteration errors
-            mask : ndarray
-                array of booleans with the same shape as ``tensor`` should be 0 where
-                the values are missing and 1 everywhere else. Note:  if tensor is
-                sparse, then mask should also be sparse with a fill value of 1 (or
-                True). Allows for missing values [2]_
-            cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
-                Stopping criterion for ALS, works if `tol` is not None. 
-                If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
-                If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
-            sparsity : float or int
-                If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
-            fixed_modes : list, default is []
-                A list of modes for which the initial value is not modified.
-                The last mode cannot be fixed due to error computation.
-            svd_mask_repeats: int
-                If using a tensor with masked values, this initializes using SVD multiple times to
-                remove the effect of these missing values on the initialization.
-            linesearch : bool, default is False
-                Whether to perform line search as proposed by Bro [3].
-
-            Returns
-            -------
-            CPTensor : (weight, factors)
-                * weights : 1D array of shape (rank, )
-                    all ones if normalize_factors is False (default), 
-                    weights of the (normalized) factors otherwise
-                * factors : List of factors of the CP decomposition element `i` is of shape
-                    (tensor.shape[i], rank)
-                * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
-
-            errors : list
-                A list of reconstruction errors at each iteration of the algorithms.
-
-            References
-            ----------
-            .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
-            SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
-
-            .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
-                    Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
-
-            .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
-                    Applications", PhD., University of Amsterdam, 1998
-        """
         self.rank = rank
         self.n_iter_max = n_iter_max
         self.tol = tol
@@ -862,6 +862,78 @@ class CP(DecompositionMixin):
 
 
 class CPNN(DecompositionMixin):
+    """Non-Negative Candecomp-Parafac decomposition via Alternating-Least Square
+
+        Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
+
+            ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
+
+        Parameters
+        ----------
+        tensor : ndarray
+        rank  : int
+            Number of components.
+        n_iter_max : int
+            Maximum number of iteration
+        init : {'svd', 'random'}, optional
+            Type of factor matrix initialization. See `initialize_factors`.
+        svd : str, default is 'numpy_svd'
+            function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
+        normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
+            of shape (rank, ), which will contain the norms of the factors
+        tol : float, optional
+            (Default: 1e-6) Relative reconstruction error tolerance. The
+            algorithm is considered to have found the global minimum when the
+            reconstruction error is less than `tol`.
+        random_state : {None, int, np.random.RandomState}
+        verbose : int, optional
+            Level of verbosity
+        return_errors : bool, optional
+            Activate return of iteration errors
+        mask : ndarray
+            array of booleans with the same shape as ``tensor`` should be 0 where
+            the values are missing and 1 everywhere else. Note:  if tensor is
+            sparse, then mask should also be sparse with a fill value of 1 (or
+            True). Allows for missing values [2]_
+        cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
+            Stopping criterion for ALS, works if `tol` is not None. 
+            If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
+            If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
+        sparsity : float or int
+            If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
+        fixed_modes : list, default is []
+            A list of modes for which the initial value is not modified.
+            The last mode cannot be fixed due to error computation.
+        svd_mask_repeats: int
+            If using a tensor with masked values, this initializes using SVD multiple times to
+            remove the effect of these missing values on the initialization.
+        linesearch : bool, default is False
+            Whether to perform line search as proposed by Bro [3].
+
+        Returns
+        -------
+        CPTensor : (weight, factors)
+            * weights : 1D array of shape (rank, )
+                all ones if normalize_factors is False (default), 
+                weights of the (normalized) factors otherwise
+            * factors : List of factors of the CP decomposition element `i` is of shape
+                (tensor.shape[i], rank)
+            * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
+
+        errors : list
+            A list of reconstruction errors at each iteration of the algorithms.
+
+        References
+        ----------
+        .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
+        SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
+
+        .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
+                Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
+
+        .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
+                Applications", PhD., University of Amsterdam, 1998
+    """
     def __init__(self, rank, n_iter_max=100, tol=1e-08, 
                  init='svd', svd='numpy_svd',
                  l2_reg=0,
@@ -874,78 +946,6 @@ class CPNN(DecompositionMixin):
                  cvg_criterion='abs_rec_error',
                  random_state=None, 
                  verbose=0):
-        """Non-Negative Candecomp-Parafac decomposition via Alternating-Least Square
-
-            Computes a rank-`rank` decomposition of `tensor` [1]_ such that,
-
-                ``tensor = [|weights; factors[0], ..., factors[-1] |]``.
-
-            Parameters
-            ----------
-            tensor : ndarray
-            rank  : int
-                Number of components.
-            n_iter_max : int
-                Maximum number of iteration
-            init : {'svd', 'random'}, optional
-                Type of factor matrix initialization. See `initialize_factors`.
-            svd : str, default is 'numpy_svd'
-                function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-            normalize_factors : if True, aggregate the weights of each factor in a 1D-tensor
-                of shape (rank, ), which will contain the norms of the factors
-            tol : float, optional
-                (Default: 1e-6) Relative reconstruction error tolerance. The
-                algorithm is considered to have found the global minimum when the
-                reconstruction error is less than `tol`.
-            random_state : {None, int, np.random.RandomState}
-            verbose : int, optional
-                Level of verbosity
-            return_errors : bool, optional
-                Activate return of iteration errors
-            mask : ndarray
-                array of booleans with the same shape as ``tensor`` should be 0 where
-                the values are missing and 1 everywhere else. Note:  if tensor is
-                sparse, then mask should also be sparse with a fill value of 1 (or
-                True). Allows for missing values [2]_
-            cvg_criterion : {'abs_rec_error', 'rec_error'}, optional
-                Stopping criterion for ALS, works if `tol` is not None. 
-                If 'rec_error',  ALS stops at current iteration if (previous rec_error - current rec_error) < tol.
-                If 'abs_rec_error', ALS terminates when |previous rec_error - current rec_error| < tol.
-            sparsity : float or int
-                If `sparsity` is not None, we approximate tensor as a sum of low_rank_component and sparse_component, where low_rank_component = cp_to_tensor((weights, factors)). `sparsity` denotes desired fraction or number of non-zero elements in the sparse_component of the `tensor`.
-            fixed_modes : list, default is []
-                A list of modes for which the initial value is not modified.
-                The last mode cannot be fixed due to error computation.
-            svd_mask_repeats: int
-                If using a tensor with masked values, this initializes using SVD multiple times to
-                remove the effect of these missing values on the initialization.
-            linesearch : bool, default is False
-                Whether to perform line search as proposed by Bro [3].
-
-            Returns
-            -------
-            CPTensor : (weight, factors)
-                * weights : 1D array of shape (rank, )
-                    all ones if normalize_factors is False (default), 
-                    weights of the (normalized) factors otherwise
-                * factors : List of factors of the CP decomposition element `i` is of shape
-                    (tensor.shape[i], rank)
-                * sparse_component : nD array of shape tensor.shape. Returns only if `sparsity` is not None.
-
-            errors : list
-                A list of reconstruction errors at each iteration of the algorithms.
-
-            References
-            ----------
-            .. [1] T.G.Kolda and B.W.Bader, "Tensor Decompositions and Applications",
-            SIAM REVIEW, vol. 51, n. 3, pp. 455-500, 2009.
-
-            .. [2] Tomasi, Giorgio, and Rasmus Bro. "PARAFAC and missing values." 
-                    Chemometrics and Intelligent Laboratory Systems 75.2 (2005): 163-180.
-
-            .. [3] R. Bro, "Multi-Way Analysis in the Food Industry: Models, Algorithms, and 
-                    Applications", PhD., University of Amsterdam, 1998
-        """
         self.rank = rank
         self.n_iter_max = n_iter_max
         self.tol = tol
@@ -995,45 +995,44 @@ class CPNN(DecompositionMixin):
         return f'Rank-{self.rank} Non-Negative CP decomposition.'
 
 
-class RandomisedCP(DecompositionMixin):
-    
+class RandomizedCP(DecompositionMixin):
+    """Randomised CP decomposition via sampled ALS
+
+    Parameters
+    ----------
+    tensor : ndarray
+    rank   : int
+            number of components
+    n_samples : int
+                number of samples per ALS step
+    n_iter_max : int
+                maximum number of iteration
+    init : {'svd', 'random'}, optional
+    svd : str, default is 'numpy_svd'
+        function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
+    tol : float, optional
+        tolerance: the algorithm stops when the variation in
+        the reconstruction error is less than the tolerance
+    max_stagnation: int, optional, default is 0
+                    if not zero, the maximum allowed number
+                    of iterations with no decrease in fit
+    random_state : {None, int, np.random.RandomState}, default is None
+    verbose : int, optional
+        level of verbosity
+
+    Returns
+    -------
+    factors : ndarray list
+            list of positive factors of the CP decomposition
+            element `i` is of shape ``(tensor.shape[i], rank)``
+
+    References
+    ----------
+    .. [3] Casey Battaglino, Grey Ballard and Tamara G. Kolda,
+    "A Practical Randomized CP Tensor Decomposition",
+    """
     def __init__(self, rank, n_samples, n_iter_max=100, init='random', svd='numpy_svd',
                        tol=10e-9, max_stagnation=20, random_state=None, verbose=1):
-        """Randomised CP decomposition via sampled ALS
-
-        Parameters
-        ----------
-        tensor : ndarray
-        rank   : int
-                number of components
-        n_samples : int
-                    number of samples per ALS step
-        n_iter_max : int
-                    maximum number of iteration
-        init : {'svd', 'random'}, optional
-        svd : str, default is 'numpy_svd'
-            function to use to compute the SVD, acceptable values in tensorly.SVD_FUNS
-        tol : float, optional
-            tolerance: the algorithm stops when the variation in
-            the reconstruction error is less than the tolerance
-        max_stagnation: int, optional, default is 0
-                        if not zero, the maximum allowed number
-                        of iterations with no decrease in fit
-        random_state : {None, int, np.random.RandomState}, default is None
-        verbose : int, optional
-            level of verbosity
-
-        Returns
-        -------
-        factors : ndarray list
-                list of positive factors of the CP decomposition
-                element `i` is of shape ``(tensor.shape[i], rank)``
-
-        References
-        ----------
-        .. [3] Casey Battaglino, Grey Ballard and Tamara G. Kolda,
-        "A Practical Randomized CP Tensor Decomposition",
-        """
         self.rank=rank
         self.n_samples=n_samples
         self.n_iter_max=n_iter_max
